/* Copyright (c) 2013 APDM
*
* longitudinal_plot.js
*
* Author: Jess VanDerwalker <jess@apdm.com>
*/

function longitudinalPlot() {
    // Physical dimentions
    var margin = { left:60, top:20, right:20, bottom:60 };
    var height = 400;
    var width = 400;

    // Scales and axis
    var xScale = d3.time.scale();
    var yScale = d3.scale.linear();
    var xAxis = d3.svg.axis().scale(xScale).orient("bottom")
        .ticks(2);
    var yAxis = d3.svg.axis().scale(yScale).orient("left")
        .ticks(2);

    // Labels
    var title = "";
    var xAxisLabel = "";
    var yAxisLabel = "";

    // Accessor functions for data
    var xValue = function(d) { return d[0]; };
    var yValue = function(d) { return d[1]; };
    var typeValue = function(d) { return d[2]; };

    // Normative values
    var normStdDev = null;
    var normMean = null;

    function longPlot(selection) {
        selection.each(function(metricData) {
            if (!metricData || !metricData.length) {
                console.warn("No data provided to longitudinal plot");
                return;
            }

            // Format the data into a nice array
            metricData = metricData.map(function(d, i) {
                return [ xValue.call(metricData, d, i),
                         yValue.call(metricData, d, i),
                         typeValue.call(metricData, d, i) ];
            });
            
            // Set ranges and domains
            var dates = metricData.map(function(d) { return d[0]; });
            // If there is only one sample, then we need to pad the
            // date range so the value is not stuck to the left
            // edge. If there are more than two, add some padding to
            // either side to get the points off the edges of the
            // plot.
            if (dates && dates.length === 1) {
                var newDates = [];
                newDates[0] = new Date(dates[0].getTime());
                newDates[1] = new Date(dates[0].getTime());
                newDates[0].setHours(newDates[0].getHours() - 1);
                newDates[1].setHours(newDates[1].getHours() + 1);
                dates = newDates;
            } else {
                dates = d3.extent(dates);
                var range = dates[1].getTime() - dates[0].getTime();
                var datePad = range * 0.10; // 10% padding
                dates[0] = new Date(dates[0].getTime() - datePad);
                dates[1] = new Date(dates[1].getTime() + datePad);
            }
            xScale.domain(dates).nice()
                .range([0, width - margin.left - margin.right]);

            var yValues = metricData.map(function(d) { return d[1]; });
            yValues = yValues.concat([normMean, normMean + (2 * normStdDev),
                            normMean - (2 * normStdDev)]);
            yScale.domain(d3.extent(yValues)).nice()
                .range([height - margin.top - margin.bottom, 0]);

            var plotSvg = d3.select(this).selectAll("svg").data([metricData]);
            var plotG = plotSvg.enter().append("svg").append("g");

            // Axis and label containers
            plotG.append("g").attr("class", "x axis");
            plotG.append("g").attr("class", "y axis");
            plotG.append("text").attr("class", "x label");
            plotG.append("text").attr("class", "y label");
            plotG.append("text").attr("class", "title label");

            // Only create the norm line and range if provided
            if (normMean) {
            plotG.append("line").attr("class", "norm_line");
                if (normStdDev) {
                    plotG.append("rect").attr("class", "norm_range_rect");
                }
            }

            // Update
            plotSvg.attr("width", width).attr("height", height);
            
            plotG = plotSvg.select("g")
                .attr("transform", "translate(" + margin.left +
                      "," + margin.top + ")");

            plotG.select(".x.axis")
                .attr("transform", "translate(0," + yScale.range()[0] + ")")
                .call(xAxis);
            plotG.select(".y.axis")
                .attr("transform", "translate(" + xScale.range()[0] + ",0)")
                .call(yAxis);

            var xLabel = plotG.select(".x.label")
                .attr("text-anchor", "middle")
                .attr("x", (width - margin.left - margin.right)/2)
                .attr("y", (height))
                .text(xAxisLabel);

            var box = xLabel.node().getBBox();
            var hOffset = (height - margin.top - margin.bottom) +
                xAxis.tickPadding() + xAxis.tickSize() + box.height + (box.height / 2);
            xLabel.attr("y", hOffset);

            plotG.select(".y.label")
                .attr("text-anchor", "middle")
                .attr("x", (margin.top + margin.bottom - height)/2)
                .attr("y", -35)
                .attr("transform", "rotate(-90)")
                .text(yAxisLabel);

            plotG.select(".title.label")
                .attr("text-anchor", "middle")
                .attr("x", (width - margin.left - margin.right)/2)
                .attr("y", -10)
                .text(title);

            // Create the symbols on the plot
            var symEnter = plotG.selectAll(".point")
                .data(metricData)
                .enter();
            symEnter.append("path")
                .attr("class", "point")
                .attr("d", d3.svg.symbol())
                .attr("transform", function(d) { return "translate(" + X(d) +
                                                 "," + Y(d) + ")"; });

            // Line for normal mean and rect for normal mean
            // range. This is currently 2 * normal standard deviation
            plotG.select(".norm_range_rect")
                .attr("x", 0)
                .attr("width", width - margin.left - margin.right)
                .attr("y", yScale(normMean + (2 * normStdDev)))
                .attr("height", yScale(normMean - (2 * normStdDev)) -
                      yScale(normMean + (2 * normStdDev)));
            
            plotG.select(".norm_line")
                .attr("x1", 0).attr("x2", width - margin.left - margin.right)
                .attr("y1", yScale(normMean)).attr("y2", yScale(normMean));
        });
    }

    function X(d) {
        return xScale(d[0]);
    }
    function Y(d) {
        return yScale(d[1]);
    }
    function TYPE(d) {
        return d[3];
    }

    // Layout
    longPlot.margin = function(d) {
        if (!arguments.length) return margin;
        margin = d;
        return longPlot;
    };
    longPlot.height = function(d) {
        if (!arguments.length) return height;
        height = d;
        return longPlot;
    };
    longPlot.width = function(d) {
        if (!arguments.length) return width;
        width = d;
        return longPlot;
    };

    // Data access
    longPlot.x = function(d) {
        if (!arguments.length) return xValue;
        xValue = d;
        return longPlot;
    };
    longPlot.y = function(d) {
        if (!arguments.length) return yValue;
        yValue = d;
        return longPlot;
    };
    longPlot.type = function(d) {
        if (!arguments.length) return typeValue;
        typeValue = d;
        return longPlot;
    };

    // Labels
    longPlot.title = function(d) {
        if (!arguments.length) return title;
        title = d;
        return longPlot;
    };
    longPlot.xAxisLabel = function(d) {
        if (!arguments.length) return xAxisLabel;
        xAxisLabel = d;
        return longPlot;
    };
    longPlot.yAxisLabel = function(d) {
        if (!arguments.length) return yAxisLabel;
        yAxisLabel = d;
        return longPlot;
    };

    // Set/get for normative data
    longPlot.normStdDev = function(d) {
        if (!arguments.length) return normStdDev;
        normStdDev = d;
        return longPlot;
    };
    longPlot.normMean = function(d) {
        if (!arguments.length) return normMean;
        normMean = d;
        return longPlot;
    };

    return longPlot;
}
